@@ -52,6 +52,7 @@ class ProtocolEnum(Enum):
     SMP_V0 = 'SMP_V0'
     SMP_M_GAS_METER_V0 = 'SMP_M_GAS_METER_V0'
     SMP_M_ENERGY_METER_V0 = 'SMP_M_ENERGY_METER_V0'
+    SMP_M_ENERGY_METER_V1 = 'SMP_M_ENERGY_METER_V1'
     SMP_M_JUPITER_08B_V0 = 'SMP_M_JUPITER_08B_V0'
     SMP_M_JUPITER_12B_V0 = 'SMP_M_JUPITER_12B_V0'
     SMP_M_JUPITER_16B_V0 = 'SMP_M_JUPITER_16B_V0'
@@ -72,6 +73,7 @@ class ProtocolEnum(Enum):
     WATER5_V_FLUO_LORA_V0 = 'WATER5_V_FLUO_LORA_V0'
     SMP_M_INTERNAL_INFO_DATA = 'SMP_M_INTERNAL_INFO_DATA'
     ARVAS_API_V0 = 'ARVAS_API_V0'
+    UNIVERSAL_API_V0 = 'UNIVERSAL_API_V0'
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
@@ -139,9 +141,29 @@ class SensorType(Enum):
     Enumeration of sensor types for devices
     """
     ENERGY_METER_VOLTAGE = 'ENERGY_METER_VOLTAGE'
+    ENERGY_METER_VOLTAGE_LINEAR = 'ENERGY_METER_VOLTAGE_LINEAR'
+    ENERGY_METER_VOLTAGE_NEUTRAL = 'ENERGY_METER_VOLTAGE_NEUTRAL'
+    ENERGY_METER_VOLTAGE_PHASE_A = 'ENERGY_METER_VOLTAGE_PHASE_A'
+    ENERGY_METER_VOLTAGE_PHASE_B = 'ENERGY_METER_VOLTAGE_PHASE_B'
+    ENERGY_METER_VOLTAGE_PHASE_C = 'ENERGY_METER_VOLTAGE_PHASE_C'
     ENERGY_METER_CURRENT = 'ENERGY_METER_CURRENT'
+    ENERGY_METER_CURRENT_LINEAR = 'ENERGY_METER_CURRENT_LINEAR'
+    ENERGY_METER_CURRENT_NEUTRAL = 'ENERGY_METER_CURRENT_NEUTRAL'
+    ENERGY_METER_CURRENT_PHASE_A = 'ENERGY_METER_CURRENT_PHASE_A'
+    ENERGY_METER_CURRENT_PHASE_B = 'ENERGY_METER_CURRENT_PHASE_B'
+    ENERGY_METER_CURRENT_PHASE_C = 'ENERGY_METER_CURRENT_PHASE_C'
     ENERGY_METER_FREQUENCY = 'ENERGY_METER_FREQUENCY'
+    ENERGY_METER_FREQUENCY_LINEAR = 'ENERGY_METER_FREQUENCY_LINEAR'
+    ENERGY_METER_FREQUENCY_NEUTRAL = 'ENERGY_METER_FREQUENCY_NEUTRAL'
+    ENERGY_METER_FREQUENCY_PHASE_A = 'ENERGY_METER_FREQUENCY_PHASE_A'
+    ENERGY_METER_FREQUENCY_PHASE_B = 'ENERGY_METER_FREQUENCY_PHASE_B'
+    ENERGY_METER_FREQUENCY_PHASE_C = 'ENERGY_METER_FREQUENCY_PHASE_C'
     ENERGY_METER_POWER_FACTOR = 'ENERGY_METER_POWER_FACTOR'
+    ENERGY_METER_POWER_FACTOR_LINEAR = 'ENERGY_METER_POWER_FACTOR_LINEAR'
+    ENERGY_METER_POWER_FACTOR_NEUTRAL = 'ENERGY_METER_POWER_FACTOR_NEUTRAL'
+    ENERGY_METER_POWER_FACTOR_PHASE_A = 'ENERGY_METER_POWER_FACTOR_PHASE_A'
+    ENERGY_METER_POWER_FACTOR_PHASE_B = 'ENERGY_METER_POWER_FACTOR_PHASE_B'
+    ENERGY_METER_POWER_FACTOR_PHASE_C = 'ENERGY_METER_POWER_FACTOR_PHASE_C'
     TEMPERATURE = 'TEMPERATURE'
     BATTERY = 'BATTERY'
     UPTIME = 'UPTIME'
@@ -170,6 +192,8 @@ class DeviceValueMarker(Enum):
     NOT_CHECKED = 'NOT_CHECKED'
     REJECTED_OVERFLOW = 'REJECTED_OVERFLOW'
     REJECTED_VALUE = 'REJECTED_VALUE'
+    VALUE_INVALID = 'VALUE_INVALID'
+    VALUE_MISSING = 'VALUE_MISSING'
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
