@@ -1,5 +1,6 @@
 import base64
 import json
+from decimal import Decimal
 from datetime import date, datetime
 from typing import Dict, Any, List, Tuple, Optional, Union
 from uuid import UUID
@@ -7,14 +8,14 @@ from uuid import UUID
 from ul_api_utils.api_resource.api_response import JsonApiResponsePayload
 from ul_api_utils.api_resource.api_response_payload_alias import ApiBaseUserModelPayloadResponse, ApiBaseModelPayloadResponse
 from ul_api_utils.internal_api.internal_api import InternalApi
-from ul_api_utils.internal_api.internal_api_response import InternalApiResponse
+from ul_api_utils.internal_api.internal_api_response import InternalApiResponse, InternalApiResponseCheckLevel
 from ul_api_utils.validators.custom_fields import WhiteSpaceStrippedStr, PgTypeInt32, CronScheduleStr, UTCOffsetSeconds
 from ul_api_utils.utils.json_encoder import CustomJSONEncoder
 from pydantic import BaseModel, UUID4, Field
 
 from data_aggregator_sdk.constants.enums import IntervalSelectValue, EncryptionType, ResourceKind, NetworkTypeEnum, \
     NetworkSysTypeEnum, DeviceHack, DataAggregatorApiUserType, DownlinkTaskType, ScheduleType, ReglamentType, \
-    SignalModulation, DownlinkTaskStatus, IntegrationV0MessageEvent, JournalDataType
+    SignalModulation, DownlinkTaskStatus, IntegrationV0MessageEvent, JournalDataType, SensorType
 from data_aggregator_sdk.data_aggregator_api_sdk_config import DataAggregatorApiSdkConfig
 from data_aggregator_sdk.integration_message import ProfileKind, ProfileGranulation
 from data_aggregator_sdk.types.device import ApiImportDeviceResponse, ApiDataGatewayNetworkDevicePayloadResponse, \
@@ -302,8 +303,7 @@ class ApiDeviceChannelResponse(BaseModel):
 
 
 class ApiDeviceNoData(JsonApiResponsePayload):
-    device_id: UUID4 = Field(title='Device Id')
-    serial_number: int = Field(title='Serial Number')
+    meter_id: UUID4 = Field(title='Meter Id')
     kind: ResourceKind = Field(title='Kind', description=f'Kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number')
     last_value: Optional[float] = Field(None, title='Last Value')
@@ -343,7 +343,7 @@ class ApiDeviceEvent(BaseModel):
 
 
 class ApiDeviceNoDataList(BaseModel):
-    devices: List[DeviceDescriptionModel] = Field(title='DeviceDescriptionModel', description='array of DeviceDescriptionModel')
+    devices: List[GeneralDeviceMeterValueDescriptionModel] = Field(title='GeneralDeviceMeterValueDescriptionModel', description='array of GeneralDeviceMeterValueDescriptionModel')
     period_from: datetime = Field(title='Period From', description='period from (datetime)')
     period_to: Optional[datetime] = Field(None, title='Period To', description='period to (datetime)')
 
@@ -509,6 +509,18 @@ class UpdateSyncOffsetResponse(JsonApiResponsePayload):
     is_confirmed: bool = Field(title='Is Confirmed', description='device state sync offset is confirmed')
 
 
+class DeviceMeterLastValuesResponse(JsonApiResponsePayload):
+    device_meter_id: UUID
+    last_value_date: datetime
+    last_value: Decimal
+
+
+class ApiDeviceMeterSyncDataResponse(JsonApiResponsePayload):
+    device_meters_values: List[DeviceMeterLastValuesResponse]
+    offset_id: Optional[UUID]
+    has_next: bool
+
+
 class ApiModificationResponse(BaseModel):
     sys_name: str = Field(title='System Name', description='system name')
     name_ru: str = Field(title='Name Ru', description='name in russian language')
@@ -675,7 +687,19 @@ class ApiDeviceUptimeResponse(JsonApiResponsePayload):
     device_id: UUID4 = Field(title='Device Identifier', description='device unique identifier')
     channel_id: UUID4 = Field(title='Device Channel Identifier', description='device channel unique identifier')
     date: datetime = Field(title='Date', description='date')
-    uptime_s: int = Field(title='Uptime value', description='uptime value seconds')
+    value: int = Field(title='Uptime value', description='uptime value seconds')
+
+
+class ApiDeviceSensorResponse(JsonApiResponsePayload):
+    id: UUID4 = Field(title='Identifier', description='Device sensor unique identifier')
+    date_created: datetime = Field(title='Date Created', description='date and time of record creation')
+    user_created_id: UUID4 = Field(title='User Created Identifier', description='api user created id')
+    device_id: UUID4 = Field(title='Device Identifier', description='device unique identifier')
+    channel_id: Optional[UUID4] = Field(title='Device Channel Identifier', description='device channel unique identifier')
+    date: datetime = Field(title='Date', description='date')
+    sensor_type: SensorType = Field(title="Type of Device Sensor", description=f"One of sensor types ({', '.join(sensor_type.name for sensor_type in SensorType)})")
+    sensor_id: int = Field(title="Sensor ID", description="Sensor ID")
+    value: Optional[float] = Field(title="Value", description="Device Sensor value")
 
 
 class DataAggregatorApiSdk:
@@ -944,7 +968,7 @@ class DataAggregatorApiSdk:
         devices: DeviceValueBatteryLvlModel,
         locf: Optional[bool],
     ) -> List[ApiDeviceLowBatteryResponse]:
-        return self._api_device.request_post('/list-device/battery-lvl-for-device-list', json=devices, q={
+        return self._api_device.request_post('/devices/device-sensors/battery-lvl-for-device-list', json=devices, q={
             'period_from': period_from,
             'period_to': period_to,
             'locf': locf,
@@ -1207,6 +1231,17 @@ class DataAggregatorApiSdk:
     def update_device_state_sync_offset(self, sync_offset_id: UUID, is_confirmed: bool = True) -> UpdateSyncOffsetResponse:
         return self._api_device.request_patch(f'/device-state-sync-offset/{str(sync_offset_id)}', json={'is_confirmed': is_confirmed}).typed(UpdateSyncOffsetResponse).check().payload
 
+    '''used in services: Iot Account Application Backend'''
+    @internal_api_error_handler
+    def get_device_meter_sync_values_chunk(self, sync_limit: int) -> ApiDeviceMeterSyncDataResponse:
+        return self._api_device.request_post('/device-meter-sync-offsets/values', q={"sync_limit": sync_limit}).typed(ApiDeviceMeterSyncDataResponse).check().payload
+
+    '''used in services: Iot Account Application Backend'''
+    @internal_api_error_handler
+    def update_device_meter_sync_offset(self, sync_offset_id: UUID, is_confirmed: bool = True) -> None:
+        self._api_device.request_patch(f'/device-meter-sync-offset/{str(sync_offset_id)}', json={'is_confirmed': is_confirmed}).check(level=InternalApiResponseCheckLevel.STATUS_CODE)
+
+
 #     '''used in services: not used'''
 #     @internal_api_error_handler
 #     def get_base_station_by_token(self, api_token: Optional[Tuple[Optional[str], str]]) -> Dict[str, Any]:  # TODO: typing
@@ -1516,6 +1551,32 @@ class DataAggregatorApiSdk:
         if offset is not None:
             query_params['offset'] = offset
         return self._api_device.request_get(
-            f'/devices/{device_id}/uptimes',
+            f'/devices/{device_id}/device-sensors/uptimes',
             q=query_params,
         ).typed(List[ApiDeviceUptimeResponse]).check()
+
+    @internal_api_error_handler
+    def get_device_sensors(
+        self,
+        limit: Optional[int],
+        offset: Optional[int],
+        device_ids: Optional[List[UUID]] = None,
+        sensor_types: Optional[List[SensorType]] = None,
+    ) -> List[ApiDeviceSensorResponse]:
+        filters = []
+        query: Dict[str, Any] = {}
+        assert self._api_device is not None
+        if sensor_types:
+            query['sensor_types'] = ', '.join(sensor_types)  # type: ignore
+        if device_ids:
+            filters.append({"name": "device_id", "op": "in", "val": tuple(device_ids)})
+        if limit is not None:
+            query['limit'] = limit
+        if offset is not None:
+            query['offset'] = offset
+        if filters:
+            filters = json.dumps(filters, cls=CustomJSONEncoder)  # type: ignore
+        return self._api_device.request_get(
+            '/devices/device-sensors',
+            q=query | {'filter': filters},
+        ).typed(List[ApiDeviceSensorResponse]).check().payload
