@@ -1,15 +1,16 @@
-from datetime import datetime
+from datetime import datetime, date, time
 from decimal import Decimal
 from enum import Enum, IntEnum, unique
 from typing import Optional, List, Tuple, Dict, Any
 
+from dateutil.relativedelta import relativedelta
 from pydantic import BaseModel, UUID4, root_validator, Field  # pylint: disable=no-name-in-module
 from ul_api_utils.errors import ValidateApiError
 from ul_api_utils.utils.api_method import ApiMethod
 from unipipeline.message.uni_message import UniMessage
 
 from data_aggregator_sdk.constants.enums import IntegrationV0MessageEvent, IntegrationV0MessageMetaBSChannelProtocol, \
-    ProtocolEnum, IntegrationV0MessageErrorType, DownlinkTaskStatus, JournalDataType, SensorType
+    ProtocolEnum, IntegrationV0MessageErrorType, DownlinkTaskStatus, JournalDataType, SensorType, DeviceValueMarker
 from data_aggregator_sdk.constants.enums import ResourceKind
 from data_aggregator_sdk.types.device import NetworkSysTypeEnum
 
@@ -43,6 +44,10 @@ class ProfileGranulation(Enum):
     SECOND_20 = "SECOND_20"
     SECOND_30 = "SECOND_30"
 
+    DAY_01 = "DAY_1"
+
+    MONTH_01 = "MONTH_1"
+
 
 class ProfileKind(Enum):
     """
@@ -56,117 +61,249 @@ class ProfileKind(Enum):
         frequency F
         point factor K
     """
+
+    # volume V
+    VOLUME_CONSUMPTION = 'VOLUME_CONSUMPTION'
+    VOLUME_CONSUMPTION_DELTA = 'VOLUME_CONSUMPTION_DELTA'
+    VOLUME_CONSUMPTION_MAX = 'VOLUME_CONSUMPTION_MAX'
+    VOLUME_CONSUMPTION_MIN = 'VOLUME_CONSUMPTION_MIN'
+    VOLUME_CONSUMPTION_AVG = 'VOLUME_CONSUMPTION_AVG'
+    VOLUME_GENERATION = 'VOLUME_GENERATION'
+    VOLUME_GENERATION_DELTA = 'VOLUME_GENERATION_DELTA'
+    VOLUME_GENERATION_MAX = 'VOLUME_GENERATION_MAX'
+    VOLUME_GENERATION_MIN = 'VOLUME_GENERATION_MIN'
+    VOLUME_GENERATION_AVG = 'VOLUME_GENERATION_AVG'
     # active generated energy A-
     ENERGY_A_N = 'ENERGY_A_N'
     ENERGY_A_N_A = 'ENERGY_A_N_A'
     ENERGY_A_N_B = 'ENERGY_A_N_B'
     ENERGY_A_N_C = 'ENERGY_A_N_C'
+    ENERGY_A_N_DELTA = 'ENERGY_A_N_DELTA'
+    ENERGY_A_N_A_DELTA = 'ENERGY_A_N_A_DELTA'
+    ENERGY_A_N_B_DELTA = 'ENERGY_A_N_B_DELTA'
+    ENERGY_A_N_C_DELTA = 'ENERGY_A_N_C_DELTA'
+    ENERGY_A_N_MAX = 'ENERGY_A_N_MAX'
+    ENERGY_A_N_A_MAX = 'ENERGY_A_N_A_MAX'
+    ENERGY_A_N_B_MAX = 'ENERGY_A_N_B_MAX'
+    ENERGY_A_N_C_MAX = 'ENERGY_A_N_C_MAX'
+    ENERGY_A_N_MIN = 'ENERGY_A_N_MIN'
+    ENERGY_A_N_A_MIN = 'ENERGY_A_N_A_MIN'
+    ENERGY_A_N_B_MIN = 'ENERGY_A_N_B_MIN'
+    ENERGY_A_N_C_MIN = 'ENERGY_A_N_C_MIN'
+    ENERGY_A_N_AVG = 'ENERGY_A_N_AVG'
+    ENERGY_A_N_A_AVG = 'ENERGY_A_N_A_AVG'
+    ENERGY_A_N_B_AVG = 'ENERGY_A_N_B_AVG'
+    ENERGY_A_N_C_AVG = 'ENERGY_A_N_C_AVG'
     # active consumed energy A+
     ENERGY_A_P = 'ENERGY_A_P'
     ENERGY_A_P_A = 'ENERGY_A_P_A'
     ENERGY_A_P_B = 'ENERGY_A_P_B'
     ENERGY_A_P_C = 'ENERGY_A_P_C'
+    ENERGY_A_P_DELTA = 'ENERGY_A_P_DELTA'
+    ENERGY_A_P_A_DELTA = 'ENERGY_A_P_A_DELTA'
+    ENERGY_A_P_B_DELTA = 'ENERGY_A_P_B_DELTA'
+    ENERGY_A_P_C_DELTA = 'ENERGY_A_P_C_DELTA'
+    ENERGY_A_P_MAX = 'ENERGY_A_P_MAX'
+    ENERGY_A_P_A_MAX = 'ENERGY_A_P_A_MAX'
+    ENERGY_A_P_B_MAX = 'ENERGY_A_P_B_MAX'
+    ENERGY_A_P_C_MAX = 'ENERGY_A_P_C_MAX'
+    ENERGY_A_P_MIN = 'ENERGY_A_P_MIN'
+    ENERGY_A_P_A_MIN = 'ENERGY_A_P_A_MIN'
+    ENERGY_A_P_B_MIN = 'ENERGY_A_P_B_MIN'
+    ENERGY_A_P_C_MIN = 'ENERGY_A_P_C_MIN'
+    ENERGY_A_P_AVG = 'ENERGY_A_P_AVG'
+    ENERGY_A_P_A_AVG = 'ENERGY_A_P_A_AVG'
+    ENERGY_A_P_B_AVG = 'ENERGY_A_P_B_AVG'
+    ENERGY_A_P_C_AVG = 'ENERGY_A_P_C_AVG'
     # reactive generated energy R-
     ENERGY_R_N = 'ENERGY_R_N'
     ENERGY_R_N_A = 'ENERGY_R_N_A'
     ENERGY_R_N_B = 'ENERGY_R_N_B'
     ENERGY_R_N_C = 'ENERGY_R_N_C'
+    ENERGY_R_N_DELTA = 'ENERGY_R_N_DELTA'
+    ENERGY_R_N_A_DELTA = 'ENERGY_R_N_A_DELTA'
+    ENERGY_R_N_B_DELTA = 'ENERGY_R_N_B_DELTA'
+    ENERGY_R_N_C_DELTA = 'ENERGY_R_N_C_DELTA'
+    ENERGY_R_N_MAX = 'ENERGY_R_N_MAX'
+    ENERGY_R_N_A_MAX = 'ENERGY_R_N_A_MAX'
+    ENERGY_R_N_B_MAX = 'ENERGY_R_N_B_MAX'
+    ENERGY_R_N_C_MAX = 'ENERGY_R_N_C_MAX'
+    ENERGY_R_N_MIN = 'ENERGY_R_N_MIN'
+    ENERGY_R_N_A_MIN = 'ENERGY_R_N_A_MIN'
+    ENERGY_R_N_B_MIN = 'ENERGY_R_N_B_MIN'
+    ENERGY_R_N_C_MIN = 'ENERGY_R_N_C_MIN'
+    ENERGY_R_N_AVG = 'ENERGY_R_N_AVG'
+    ENERGY_R_N_A_AVG = 'ENERGY_R_N_A_AVG'
+    ENERGY_R_N_B_AVG = 'ENERGY_R_N_B_AVG'
+    ENERGY_R_N_C_AVG = 'ENERGY_R_N_C_AVG'
     # reactive consumed energy R+
     ENERGY_R_P = 'ENERGY_R_P'
     ENERGY_R_P_A = 'ENERGY_R_P_A'
     ENERGY_R_P_B = 'ENERGY_R_P_B'
     ENERGY_R_P_C = 'ENERGY_R_P_C'
+    ENERGY_R_P_DELTA = 'ENERGY_R_P_DELTA'
+    ENERGY_R_P_A_DELTA = 'ENERGY_R_P_A_DELTA'
+    ENERGY_R_P_B_DELTA = 'ENERGY_R_P_B_DELTA'
+    ENERGY_R_P_C_DELTA = 'ENERGY_R_P_C_DELTA'
+    ENERGY_R_P_MAX = 'ENERGY_R_P_MAX'
+    ENERGY_R_P_A_MAX = 'ENERGY_R_P_A_MAX'
+    ENERGY_R_P_B_MAX = 'ENERGY_R_P_B_MAX'
+    ENERGY_R_P_C_MAX = 'ENERGY_R_P_C_MAX'
+    ENERGY_R_P_MIN = 'ENERGY_R_P_MIN'
+    ENERGY_R_P_A_MIN = 'ENERGY_R_P_A_MIN'
+    ENERGY_R_P_B_MIN = 'ENERGY_R_P_B_MIN'
+    ENERGY_R_P_C_MIN = 'ENERGY_R_P_C_MIN'
+    ENERGY_R_P_AVG = 'ENERGY_R_P_AVG'
+    ENERGY_R_P_A_AVG = 'ENERGY_R_P_A_AVG'
+    ENERGY_R_P_B_AVG = 'ENERGY_R_P_B_AVG'
+    ENERGY_R_P_C_AVG = 'ENERGY_R_P_C_AVG'
     # full power S
     FULL_POWER_ABC = 'FULL_POWER_ABC'
     FULL_POWER_A = 'FULL_POWER_A'
     FULL_POWER_B = 'FULL_POWER_B'
     FULL_POWER_C = 'FULL_POWER_C'
-    FULL_POWER_MIN_ABC = 'FULL_POWER_MIN_ABC'
-    FULL_POWER_MIN_A = 'FULL_POWER_MIN_A'
-    FULL_POWER_MIN_B = 'FULL_POWER_MIN_B'
-    FULL_POWER_MIN_C = 'FULL_POWER_MIN_C'
+    FULL_POWER_DELTA_ABC = 'FULL_POWER_DELTA_ABC'
+    FULL_POWER_DELTA_A = 'FULL_POWER_DELTA_A'
+    FULL_POWER_DELTA_B = 'FULL_POWER_DELTA_B'
+    FULL_POWER_DELTA_C = 'FULL_POWER_DELTA_C'
     FULL_POWER_MAX_ABC = 'FULL_POWER_MAX_ABC'
     FULL_POWER_MAX_A = 'FULL_POWER_MAX_A'
     FULL_POWER_MAX_B = 'FULL_POWER_MAX_B'
     FULL_POWER_MAX_C = 'FULL_POWER_MAX_C'
+    FULL_POWER_MIN_ABC = 'FULL_POWER_MIN_ABC'
+    FULL_POWER_MIN_A = 'FULL_POWER_MIN_A'
+    FULL_POWER_MIN_B = 'FULL_POWER_MIN_B'
+    FULL_POWER_MIN_C = 'FULL_POWER_MIN_C'
+    FULL_POWER_AVG_ABC = 'FULL_POWER_AVG_ABC'
+    FULL_POWER_AVG_A = 'FULL_POWER_AVG_A'
+    FULL_POWER_AVG_B = 'FULL_POWER_AVG_B'
+    FULL_POWER_AVG_C = 'FULL_POWER_AVG_C'
     # active power P
     ACTIVE_POWER_ABC = 'ACTIVE_POWER_ABC'
     ACTIVE_POWER_A = 'ACTIVE_POWER_A'
     ACTIVE_POWER_B = 'ACTIVE_POWER_B'
     ACTIVE_POWER_C = 'ACTIVE_POWER_C'
-    ACTIVE_POWER_MIN_ABC = 'ACTIVE_POWER_MIN_ABC'
-    ACTIVE_POWER_MIN_A = 'ACTIVE_POWER_MIN_A'
-    ACTIVE_POWER_MIN_B = 'ACTIVE_POWER_MIN_B'
-    ACTIVE_POWER_MIN_C = 'ACTIVE_POWER_MIN_C'
+    ACTIVE_POWER_DELTA_ABC = 'ACTIVE_POWER_DELTA_ABC'
+    ACTIVE_POWER_DELTA_A = 'ACTIVE_POWER_DELTA_A'
+    ACTIVE_POWER_DELTA_B = 'ACTIVE_POWER_DELTA_B'
+    ACTIVE_POWER_DELTA_C = 'ACTIVE_POWER_DELTA_C'
     ACTIVE_POWER_MAX_ABC = 'ACTIVE_POWER_MAX_ABC'
     ACTIVE_POWER_MAX_A = 'ACTIVE_POWER_MAX_A'
     ACTIVE_POWER_MAX_B = 'ACTIVE_POWER_MAX_B'
     ACTIVE_POWER_MAX_C = 'ACTIVE_POWER_MAX_C'
+    ACTIVE_POWER_MIN_ABC = 'ACTIVE_POWER_MIN_ABC'
+    ACTIVE_POWER_MIN_A = 'ACTIVE_POWER_MIN_A'
+    ACTIVE_POWER_MIN_B = 'ACTIVE_POWER_MIN_B'
+    ACTIVE_POWER_MIN_C = 'ACTIVE_POWER_MIN_C'
+    ACTIVE_POWER_AVG_ABC = 'ACTIVE_POWER_AVG_ABC'
+    ACTIVE_POWER_AVG_A = 'ACTIVE_POWER_AVG_A'
+    ACTIVE_POWER_AVG_B = 'ACTIVE_POWER_AVG_B'
+    ACTIVE_POWER_AVG_C = 'ACTIVE_POWER_AVG_C'
     # reactive power Q
     REACTIVE_POWER_ABC = 'REACTIVE_POWER_ABC'
     REACTIVE_POWER_A = 'REACTIVE_POWER_A'
     REACTIVE_POWER_B = 'REACTIVE_POWER_B'
     REACTIVE_POWER_C = 'REACTIVE_POWER_C'
-    REACTIVE_POWER_MIN_ABC = 'REACTIVE_POWER_MIN_ABC'
-    REACTIVE_POWER_MIN_A = 'REACTIVE_POWER_MIN_A'
-    REACTIVE_POWER_MIN_B = 'REACTIVE_POWER_MIN_B'
-    REACTIVE_POWER_MIN_C = 'REACTIVE_POWER_MIN_C'
+    REACTIVE_POWER_DELTA_ABC = 'REACTIVE_POWER_DELTA_ABC'
+    REACTIVE_POWER_DELTA_A = 'REACTIVE_POWER_DELTA_A'
+    REACTIVE_POWER_DELTA_B = 'REACTIVE_POWER_DELTA_B'
+    REACTIVE_POWER_DELTA_C = 'REACTIVE_POWER_DELTA_C'
     REACTIVE_POWER_MAX_ABC = 'REACTIVE_POWER_MAX_ABC'
     REACTIVE_POWER_MAX_A = 'REACTIVE_POWER_MAX_A'
     REACTIVE_POWER_MAX_B = 'REACTIVE_POWER_MAX_B'
     REACTIVE_POWER_MAX_C = 'REACTIVE_POWER_MAX_C'
+    REACTIVE_POWER_MIN_ABC = 'REACTIVE_POWER_MIN_ABC'
+    REACTIVE_POWER_MIN_A = 'REACTIVE_POWER_MIN_A'
+    REACTIVE_POWER_MIN_B = 'REACTIVE_POWER_MIN_B'
+    REACTIVE_POWER_MIN_C = 'REACTIVE_POWER_MIN_C'
+    REACTIVE_POWER_AVG_ABC = 'REACTIVE_POWER_AVG_ABC'
+    REACTIVE_POWER_AVG_A = 'REACTIVE_POWER_AVG_A'
+    REACTIVE_POWER_AVG_B = 'REACTIVE_POWER_AVG_B'
+    REACTIVE_POWER_AVG_C = 'REACTIVE_POWER_AVG_C'
     # voltage U
     VOLTAGE_ABC = 'VOLTAGE_ABC'
     VOLTAGE_A = 'VOLTAGE_A'
     VOLTAGE_B = 'VOLTAGE_B'
     VOLTAGE_C = 'VOLTAGE_C'
-    VOLTAGE_MIN_ABC = 'VOLTAGE_MIN_ABC'
-    VOLTAGE_MIN_A = 'VOLTAGE_MIN_A'
-    VOLTAGE_MIN_B = 'VOLTAGE_MIN_B'
-    VOLTAGE_MIN_C = 'VOLTAGE_MIN_C'
+    VOLTAGE_DELTA_ABC = 'VOLTAGE_DELTA_ABC'
+    VOLTAGE_DELTA_A = 'VOLTAGE_DELTA_A'
+    VOLTAGE_DELTA_B = 'VOLTAGE_DELTA_B'
+    VOLTAGE_DELTA_C = 'VOLTAGE_DELTA_C'
     VOLTAGE_MAX_ABC = 'VOLTAGE_MAX_ABC'
     VOLTAGE_MAX_A = 'VOLTAGE_MAX_A'
     VOLTAGE_MAX_B = 'VOLTAGE_MAX_B'
     VOLTAGE_MAX_C = 'VOLTAGE_MAX_C'
+    VOLTAGE_MIN_ABC = 'VOLTAGE_MIN_ABC'
+    VOLTAGE_MIN_A = 'VOLTAGE_MIN_A'
+    VOLTAGE_MIN_B = 'VOLTAGE_MIN_B'
+    VOLTAGE_MIN_C = 'VOLTAGE_MIN_C'
+    VOLTAGE_AVG_ABC = 'VOLTAGE_AVG_ABC'
+    VOLTAGE_AVG_A = 'VOLTAGE_AVG_A'
+    VOLTAGE_AVG_B = 'VOLTAGE_AVG_B'
+    VOLTAGE_AVG_C = 'VOLTAGE_AVG_C'
     # current I
     CURRENT_ABC = 'CURRENT_ABC'
     CURRENT_A = 'CURRENT_A'
     CURRENT_B = 'CURRENT_B'
     CURRENT_C = 'CURRENT_C'
-    CURRENT_MIN_ABC = 'CURRENT_MIN_ABC'
-    CURRENT_MIN_A = 'CURRENT_MIN_A'
-    CURRENT_MIN_B = 'CURRENT_MIN_B'
-    CURRENT_MIN_C = 'CURRENT_MIN_C'
+    CURRENT_DELTA_ABC = 'CURRENT_DELTA_ABC'
+    CURRENT_DELTA_A = 'CURRENT_DELTA_A'
+    CURRENT_DELTA_B = 'CURRENT_DELTA_B'
+    CURRENT_DELTA_C = 'CURRENT_DELTA_C'
     CURRENT_MAX_ABC = 'CURRENT_MAX_ABC'
     CURRENT_MAX_A = 'CURRENT_MAX_A'
     CURRENT_MAX_B = 'CURRENT_MAX_B'
     CURRENT_MAX_C = 'CURRENT_MAX_C'
+    CURRENT_MIN_ABC = 'CURRENT_MIN_ABC'
+    CURRENT_MIN_A = 'CURRENT_MIN_A'
+    CURRENT_MIN_B = 'CURRENT_MIN_B'
+    CURRENT_MIN_C = 'CURRENT_MIN_C'
+    CURRENT_AVG_ABC = 'CURRENT_AVG_ABC'
+    CURRENT_AVG_A = 'CURRENT_AVG_A'
+    CURRENT_AVG_B = 'CURRENT_AVG_B'
+    CURRENT_AVG_C = 'CURRENT_AVG_C'
     # frequency F
     FREQUENCY_ABC = 'FREQUENCY_ABC'
     FREQUENCY_A = 'FREQUENCY_A'
     FREQUENCY_B = 'FREQUENCY_B'
     FREQUENCY_C = 'FREQUENCY_C'
-    FREQUENCY_MIN_ABC = 'FREQUENCY_MIN_ABC'
-    FREQUENCY_MIN_A = 'FREQUENCY_MIN_A'
-    FREQUENCY_MIN_B = 'FREQUENCY_MIN_B'
-    FREQUENCY_MIN_C = 'FREQUENCY_MIN_C'
+    FREQUENCY_DELTA_ABC = 'FREQUENCY_DELTA_ABC'
+    FREQUENCY_DELTA_A = 'FREQUENCY_DELTA_A'
+    FREQUENCY_DELTA_B = 'FREQUENCY_DELTA_B'
+    FREQUENCY_DELTA_C = 'FREQUENCY_DELTA_C'
     FREQUENCY_MAX_ABC = 'FREQUENCY_MAX_ABC'
     FREQUENCY_MAX_A = 'FREQUENCY_MAX_A'
     FREQUENCY_MAX_B = 'FREQUENCY_MAX_B'
     FREQUENCY_MAX_C = 'FREQUENCY_MAX_C'
+    FREQUENCY_MIN_ABC = 'FREQUENCY_MIN_ABC'
+    FREQUENCY_MIN_A = 'FREQUENCY_MIN_A'
+    FREQUENCY_MIN_B = 'FREQUENCY_MIN_B'
+    FREQUENCY_MIN_C = 'FREQUENCY_MIN_C'
+    FREQUENCY_AVG_ABC = 'FREQUENCY_AVG_ABC'
+    FREQUENCY_AVG_A = 'FREQUENCY_AVG_A'
+    FREQUENCY_AVG_B = 'FREQUENCY_AVG_B'
+    FREQUENCY_AVG_C = 'FREQUENCY_AVG_C'
     # power factor K
     POWER_FACTOR_ABC = 'POWER_FACTOR_ABC'
     POWER_FACTOR_A = 'POWER_FACTOR_A'
     POWER_FACTOR_B = 'POWER_FACTOR_B'
     POWER_FACTOR_C = 'POWER_FACTOR_C'
-    POWER_FACTOR_MIN_ABC = 'POWER_FACTOR_MIN_ABC'
-    POWER_FACTOR_MIN_A = 'POWER_FACTOR_MIN_A'
-    POWER_FACTOR_MIN_B = 'POWER_FACTOR_MIN_B'
-    POWER_FACTOR_MIN_C = 'POWER_FACTOR_MIN_C'
+    POWER_FACTOR_DELTA_ABC = 'POWER_FACTOR_DELTA_ABC'
+    POWER_FACTOR_DELTA_A = 'POWER_FACTOR_DELTA_A'
+    POWER_FACTOR_DELTA_B = 'POWER_FACTOR_DELTA_B'
+    POWER_FACTOR_DELTA_C = 'POWER_FACTOR_DELTA_C'
     POWER_FACTOR_MAX_ABC = 'POWER_FACTOR_MAX_ABC'
     POWER_FACTOR_MAX_A = 'POWER_FACTOR_MAX_A'
     POWER_FACTOR_MAX_B = 'POWER_FACTOR_MAX_B'
     POWER_FACTOR_MAX_C = 'POWER_FACTOR_MAX_C'
+    POWER_FACTOR_MIN_ABC = 'POWER_FACTOR_MIN_ABC'
+    POWER_FACTOR_MIN_A = 'POWER_FACTOR_MIN_A'
+    POWER_FACTOR_MIN_B = 'POWER_FACTOR_MIN_B'
+    POWER_FACTOR_MIN_C = 'POWER_FACTOR_MIN_C'
+    POWER_FACTOR_AVG_ABC = 'POWER_FACTOR_AVG_ABC'
+    POWER_FACTOR_AVG_A = 'POWER_FACTOR_AVG_A'
+    POWER_FACTOR_AVG_B = 'POWER_FACTOR_AVG_B'
+    POWER_FACTOR_AVG_C = 'POWER_FACTOR_AVG_C'
 
     def __repr__(self) -> str:
         return f'{type(self).__name__}.{self.name}'
@@ -191,6 +328,21 @@ class CounterType(Enum):
     ENERGY_PHASE_C = 'ENERGY_PHASE_C'
 
 
+class ValidationStatus(Enum):
+    """
+    Enumeration of value statuses
+    """
+    VALID = 'VALID'
+    INVALID = 'INVALID'
+    MISSING = 'MISSING'
+
+
+VALIDATION_STATUS_VALUE_MARKER_MAP = {
+    ValidationStatus.INVALID: DeviceValueMarker.VALUE_INVALID,
+    ValidationStatus.MISSING: DeviceValueMarker.VALUE_MISSING,
+}
+
+
 class IntegrationV0MessageClock(BaseModel):
     clock_id: int = Field(
         -1,
@@ -229,37 +381,62 @@ class IntegrationV0MessageRelay(BaseModel):
 
 MINUTES_IN_DAY_CONST = 24 * 60
 SECONDS_IN_DAY_CONST = MINUTES_IN_DAY_CONST * 60
-
+DAYS_IN_YEAR_CONST = 365
+MONTHS_IN_YEAR_CONST = 12
 
 GRANULATION_TO_SECONDS_DELTA_MAP = {
-    ProfileGranulation.MINUTE_01: 60,
-    ProfileGranulation.MINUTE_02: 120,
-    ProfileGranulation.MINUTE_03: 180,
-    ProfileGranulation.MINUTE_04: 240,
-    ProfileGranulation.MINUTE_05: 300,
-    ProfileGranulation.MINUTE_06: 360,
-    ProfileGranulation.MINUTE_10: 600,
-    ProfileGranulation.MINUTE_12: 720,
-    ProfileGranulation.MINUTE_15: 900,
-    ProfileGranulation.MINUTE_20: 1200,
-    ProfileGranulation.MINUTE_30: 1800,
-    ProfileGranulation.MINUTE_60: 3600,
+    ProfileGranulation.MINUTE_01: relativedelta(minutes=1, second=0),
+    ProfileGranulation.MINUTE_02: relativedelta(minutes=2, second=0),
+    ProfileGranulation.MINUTE_03: relativedelta(minutes=3, second=0),
+    ProfileGranulation.MINUTE_04: relativedelta(minutes=4, second=0),
+    ProfileGranulation.MINUTE_05: relativedelta(minutes=5, second=0),
+    ProfileGranulation.MINUTE_06: relativedelta(minutes=6, second=0),
+    ProfileGranulation.MINUTE_10: relativedelta(minutes=10, second=0),
+    ProfileGranulation.MINUTE_12: relativedelta(minutes=12, second=0),
+    ProfileGranulation.MINUTE_15: relativedelta(minutes=15, second=0),
+    ProfileGranulation.MINUTE_20: relativedelta(minutes=20, second=0),
+    ProfileGranulation.MINUTE_30: relativedelta(minutes=30, second=0),
+    ProfileGranulation.MINUTE_60: relativedelta(minutes=60, second=0),
 
-    ProfileGranulation.SECOND_01: 1,
-    ProfileGranulation.SECOND_02: 2,
-    ProfileGranulation.SECOND_03: 3,
-    ProfileGranulation.SECOND_04: 4,
-    ProfileGranulation.SECOND_05: 5,
-    ProfileGranulation.SECOND_06: 6,
-    ProfileGranulation.SECOND_10: 10,
-    ProfileGranulation.SECOND_12: 12,
-    ProfileGranulation.SECOND_15: 15,
-    ProfileGranulation.SECOND_20: 20,
-    ProfileGranulation.SECOND_30: 30,
+    ProfileGranulation.SECOND_01: relativedelta(seconds=1),
+    ProfileGranulation.SECOND_02: relativedelta(seconds=2),
+    ProfileGranulation.SECOND_03: relativedelta(seconds=3),
+    ProfileGranulation.SECOND_04: relativedelta(seconds=4),
+    ProfileGranulation.SECOND_05: relativedelta(seconds=5),
+    ProfileGranulation.SECOND_06: relativedelta(seconds=6),
+    ProfileGranulation.SECOND_10: relativedelta(seconds=10),
+    ProfileGranulation.SECOND_12: relativedelta(seconds=12),
+    ProfileGranulation.SECOND_15: relativedelta(seconds=15),
+    ProfileGranulation.SECOND_20: relativedelta(seconds=20),
+    ProfileGranulation.SECOND_30: relativedelta(seconds=30),
+
+    ProfileGranulation.DAY_01: relativedelta(days=1, hour=0, minute=0, second=0),
+
+    ProfileGranulation.MONTH_01: relativedelta(months=1, day=1, hour=0, minute=0, second=0),
 }
 
 assert len(GRANULATION_TO_SECONDS_DELTA_MAP.keys()) == len(ProfileGranulation)
 
+
+GRANULATION_TO_END_OF_DATETIME_MAP = {
+    ProfileGranulation.MINUTE_01: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute, second=0)),
+    ProfileGranulation.MINUTE_02: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 2 * 2, second=0)),
+    ProfileGranulation.MINUTE_03: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 3 * 3, second=0)),
+    ProfileGranulation.MINUTE_04: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 4 * 4, second=0)),
+    ProfileGranulation.MINUTE_05: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 5 * 5, second=0)),
+    ProfileGranulation.MINUTE_06: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 6 * 6, second=0)),
+    ProfileGranulation.MINUTE_10: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 10 * 10, second=0)),
+    ProfileGranulation.MINUTE_12: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 12 * 12, second=0)),
+    ProfileGranulation.MINUTE_15: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 15 * 15, second=0)),
+    ProfileGranulation.MINUTE_20: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 20 * 20, second=0)),
+    ProfileGranulation.MINUTE_30: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=dt.minute // 30 * 30, second=0)),
+    ProfileGranulation.MINUTE_60: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=dt.hour, minute=0, second=0)),
+
+    ProfileGranulation.DAY_01: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=dt.day), time(hour=0, minute=0, second=0)),
+
+    ProfileGranulation.MONTH_01: lambda dt: datetime.combine(date(year=dt.year, month=dt.month, day=1), time(hour=0, minute=0, second=0)),
+}
+
 GRANULATION_LENGTH_MAP = {
     ProfileGranulation.MINUTE_01: MINUTES_IN_DAY_CONST,
     ProfileGranulation.MINUTE_02: MINUTES_IN_DAY_CONST / 2,
@@ -285,8 +462,13 @@ GRANULATION_LENGTH_MAP = {
     ProfileGranulation.SECOND_15: SECONDS_IN_DAY_CONST / 15,
     ProfileGranulation.SECOND_20: SECONDS_IN_DAY_CONST / 20,
     ProfileGranulation.SECOND_30: SECONDS_IN_DAY_CONST / 30,
+
+    ProfileGranulation.DAY_01: DAYS_IN_YEAR_CONST,
+
+    ProfileGranulation.MONTH_01: MONTHS_IN_YEAR_CONST,
 }
 
+
 assert len(GRANULATION_LENGTH_MAP.keys()) == len(ProfileGranulation)
 
 
@@ -295,6 +477,11 @@ class IntegrationV0MessageProfile(BaseModel):
         title="Profile type",
         description="Profile type is like side view of collecting data for device (ex. 'VOLTAGE_ABC', 'ENERGY_A_N')",
     )
+    tariff: int = Field(
+        -1,
+        title="Profile tariff",
+        description="Profile tariff",
+    )
     granulation: ProfileGranulation = Field(
         title="Profile granulation",
         description="Granulation is detailing of collecting data, resolution of graph by device (ex. 1 hour (MINUTE_60), 1 second (SECONDS_01))",
@@ -315,6 +502,11 @@ class IntegrationV0MessageProfile(BaseModel):
 
 
 class IntegrationV0MessageConsumption(BaseModel):
+    validation_status: ValidationStatus = Field(
+        ValidationStatus.VALID,
+        title="Validation status",
+        description="Validation status integration message data field",
+    )
     tariff: int = Field(
         -1,
         title="Consumption tariff",
@@ -350,6 +542,11 @@ class IntegrationV0MessageConsumption(BaseModel):
 
 
 class IntegrationV0MessageGeneration(BaseModel):
+    validation_status: ValidationStatus = Field(
+        ValidationStatus.VALID,
+        title="Validation status",
+        description="Validation status integration message data field",
+    )
     tariff: int = Field(
         -1,
         title="Generation tariff",
@@ -363,7 +560,11 @@ class IntegrationV0MessageGeneration(BaseModel):
         title="Resource type",
         description="Generation resource type",
     )
-
+    channel: Optional[int] = Field(
+        None,
+        title="Channel",
+        description="Generation channel number for collecting info",
+    )
     value: Decimal = Field(
         title="Generation value",
         description="Generation channel value",
@@ -569,11 +770,6 @@ class IntegrationV0MessageSensor(BaseModel):
         title="Channel ID",
         description="Device channel Id",
     )
-    meter_resource_kind: Optional[ResourceKind] = Field(
-        None,
-        title="Meter Resource Kind",
-        description="Device Meter Resource Kind",
-    )
     sensor_id: int = Field(
         -1,
         title="Sensor id",
@@ -943,6 +1139,10 @@ class IntegrationV0MessageGateway(UniMessage):
     )
 
 
+class IntegrationV0MessageGatewayList(UniMessage):
+    messages: List[IntegrationV0MessageGateway]
+
+
 INTEGRATION_MESSAGE_RESOURCE_TYPE_AND_COUNTER_TYPE__TO__CONSUMED_RESOURCE_KIND: Dict[Tuple[ResourceType, CounterType], ResourceKind] = {
     (ResourceType.COMMON, CounterType.ENERGY_PHASE_A): ResourceKind.PHASE_A_ACTIVE_CONSUMED,  # TODO: for uiversal api tmp resolution
     (ResourceType.COMMON, CounterType.ENERGY_PHASE_B): ResourceKind.PHASE_B_ACTIVE_CONSUMED,  # TODO: for uiversal api tmp resolution
